(*********************************************
TLookupTable->TComponent

Provides lookup tables to common math routines.
Looking up these values from a table is much
faster than issuing the function calls directly.

The following tables are provided as direct-access
arrays to increase access speed:

lkupSin: Sin
lkupCos: CoSine
lkupArcTan: ArcTangent
*********************************************)

unit Lkuptabl;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

const
  PI = 3.14159;

type
  TLookupTable = class(TComponent)
  private
  protected
  public
     lkupSin: array[0..360] of single;
     lkupCos: array[0..360] of single;
     lkupArcTan: array[0..360] of single;
     constructor Create( AOwner: TComponent ); override;
  published
  end;

procedure Register;

implementation

constructor TLookupTable.Create( AOwner: TComponent );
var
  k: integer;
begin
  inherited Create( AOwner );
  for k := 0 to 360 do
  begin
     lkupSin[k] := Cos( k * PI / 180.0 );
     lkupCos[k] := Sin( k * PI / 180.0 );
     lkupArcTan[k] := ArcTan( k * PI / 180.0 );
  end;
end;

procedure Register;
begin
  RegisterComponents( 'NonVis', [TLookupTable] );
end;

end.
 