unit Utility;

interface

uses
	Forms, WinTypes, Graphics, SysUtils, WinProcs, MMSystem, StdCtrls, Classes,
  ExtCtrls, Messages;

const

{ Colors }
  clLightBlue = TColor($FFFF80);
  clTrans = 8421376;          { Standard transparent color }

function AppPath: String;
procedure CreateBitmapMasks( bmSource, bm1, bm2: TBitmap; clTrans: TColor );
procedure Delay( nSeconds: integer );
function Distance( const pt1, pt2: TPoint ) : real;
procedure FormatMemo( memo: TMemo );
function GetDeviceColors( h: HDC ): longint;
function GetToken( var sString: String; const sDelim: String ): String;
procedure MicroDelay( nMicroSeconds: integer );
procedure PlayWave( const sFile: String; bOption: boolean );
procedure SendClick( hWnd: THandle );
function TagOf( frm: TForm; const i: integer ): TComponent;
function RandomizeValue( const x, xRandomness: extended ): extended;
procedure WMPaletteChanged( var Msg: TWMPaletteChanged; f: TForm; bm: TBitmap );
procedure WMQueryNewPalette( var Msg: TWMQueryNewPalette; f: TForm; bm: TBitmap );

implementation

(*********************************************
Return the application path minus file name.
*********************************************)
function AppPath: String;
var
  sTemp, sPath: String;
  nPos: integer;
begin
  sTemp := Application.ExeName;
  sPath := '';
  nPos := Pos( '\', sTemp );
  while ( nPos > 0 ) do
     begin
        sPath := sPath + GetToken( sTemp, '\' ) + '\';
        nPos := Pos( '\', sTemp );
     end;
  AppPath := sPath;
end;

(*********************************************
Produce the 2 mask bitmaps for animation.
*********************************************)
procedure CreateBitmapMasks( bmSource, bm1, bm2: TBitmap; clTrans: TColor );
var
  x, y: integer;
begin
  bm1.Assign( bmSource );
  bm2.Assign( bmSource );
  for x := 0 to bmSource.Width - 1 do
     for y := 0 to bmSource.Height - 1 do
        begin
           if bmSource.Canvas.Pixels[x, y] = clTrans then
              begin
                 bm1.Canvas.Pixels[x, y] := clWhite;
                 bm2.Canvas.Pixels[x, y] := clBlack;
              end
           else
              begin
                 bm1.Canvas.Pixels[x, y] := clBlack;
              end;
        end;
end;

(*********************************************
Delay for the specified number of seconds.
*********************************************)
procedure Delay( nSeconds: integer );
var
  nHour, nMinute, nSecond, nMicro: word;
  nSecCheck: word;
begin
  DecodeTime( Now, nHour, nMinute, nSecond, nMicro );
  nSecond := nSecond + nSeconds;
  while ( nSecond >= 60 ) do
     nSecond := nSecond - 60;
  nSecCheck := 60;
  while ( nSecCheck <> nSecond ) do
     DecodeTime( Now, nHour, nMinute, nSecCheck, nMicro );
end;

(*********************************************
Standard distance formula.
*********************************************)
function Distance( const pt1, pt2: TPoint ) : real;
begin
  Distance := Sqrt( Sqr( pt2.X - pt1.X ) + Sqr( pt2.Y - pt1.Y ) )
end;

(*********************************************
Remove hard returns from a memo control, formating
the text to fit into the dimensions of the control.
*********************************************)
procedure FormatMemo( memo: TMemo );
var
  pBuf, pTest: PChar;
  nSize: integer;
  i: integer;
begin
  memo.Lines.BeginUpdate;
  nSize := memo.GetTextLen;
  pBuf := StrAlloc( nSize + 1 );
  memo.GetTextBuf( pBuf, nSize );
  for i := 0 to nSize - 1 do
     begin
        pTest := pBuf + i;
        if ( pTest^ = #10 ) or ( pTest^ = #13 ) then
           pTest^ := ' ';
     end;
  memo.SetTextBuf( pBuf );
  StrDispose( pBuf );
  memo.Lines.EndUpdate;
end;

(*********************************************
Find out how many colors the screen supports.
*********************************************)
function GetDeviceColors( h: HDC ): longint;
var
  n1, n2: longint;
begin
  n1 := GetDeviceCaps( h, PLANES );
  n2 := GetDeviceCaps( h, BITSPIXEL );
  Result := longint( 1 ) shl ( n1 * n2 );
end;

(*********************************************
Strips a token from a string.
*********************************************)
function GetToken( var sString: String; const sDelim: String ): String;
var
  nPos: integer;
begin
  nPos := Pos( sDelim, sString );
  if nPos > 0 then
     begin
        GetToken := Copy( sString, 1, nPos - 1 );
        sString := Copy( sString, nPos + 1, Length( sString ) - nPos );
     end
  else
     begin
        GetToken := sString;
        sString := '';
     end;
end;

(*********************************************
Delay for the specified number of microseconds.
*********************************************)
procedure MicroDelay( nMicroSeconds: integer );
var
  nHour, nMinute, nSecond, nMicro, nMicroFirst: word;
  nMicroCheck: word;
begin
  DecodeTime( Now, nHour, nMinute, nSecond, nMicroFirst );
  nMicroCheck := nMicroFirst + nMicroSeconds;
  nMicro := 0;
  while ( nMicro < nMicroCheck ) do
     begin
        DecodeTime( Now, nHour, nMinute, nSecond, nMicro );
        if nMicro < nMicroFirst then
           nMicro := nMicro + 1000;
     end;
end;

(*********************************************
Play a WAV file.
*********************************************)
procedure PlayWave( const sFile: String; bOption: boolean );
var
  lpFile: PChar;
  nFlags: word;
  sFile_: string;
begin
  if bOption then
     begin
        sFile_ := sFile;
        if Pos( '\', sFile ) = 0 then
           sFile_ := AppPath + sFile;
        lpFile := StrAlloc( Length( sFile_ ) + 1 );
        StrPCopy( lpFile, sFile_ );
        nFlags := SND_ASYNC + SND_NODEFAULT;
        sndPlaySound( lpFile, nFlags );
        StrDispose( lpFile );
     end;
end;

(*********************************************
Is a point in rectangle?
*********************************************)
function PointInRect( const rectTest: TRect; X, Y: integer ): boolean;
begin
  Result := ( ( X >= rectTest.Left ) and ( X <= rectTest.Right ) and
     ( Y >= rectTest.Top ) and ( Y <= rectTest.Bottom ) );
end;

(*********************************************
Randomize a float by a specified factor.
Randomness = 0 (No Randomness) to 1 (Full Randomness)
*********************************************)
function RandomizeValue( const x, xRandomness: extended ): extended;
var
  xMult, xTerm1, xTerm2: extended;
begin
  xMult := Random + 0.5;
  xTerm1 := x * xMult;
  xTerm1 := xTerm1 * xRandomness;
  xTerm2 := x * ( 1.0 - xRandomness );
  Result := xTerm1 + xTerm2;
end;

(*********************************************
Send a click message
*********************************************)
procedure SendClick( hWnd: THandle );
begin
  PostMessage( hWnd, wm_LButtonDown, 0, 0 );
  PostMessage( hWnd, wm_LButtonUp, 0, 0 );
end;

(*********************************************
Return the control with the specified tag.
*********************************************)
function TagOf( frm: TForm; const i: integer ): TComponent;
var
  j: integer;
begin
  Result := nil;
  for j := 0 to frm.ComponentCount - 1 do
     if frm.Components[j].Tag = i then
        begin
           Result := frm.Components[j];
           Break;
        end;
end;

(*********************************************
Stock functions that respsond to Windows
palette messages
*********************************************)
procedure WMPaletteChanged( var Msg: TWMPaletteChanged; f: TForm; bm: TBitmap );
var
  DC: HDC;
  palTemp: HPalette;
begin
  if Msg.PalChg = f.Handle then
     Msg.Result := 0
  else
     begin
        DC := f.Canvas.Handle;
        palTemp := SelectPalette( DC, bm.Palette, TRUE );
        RealizePalette( DC );
        UpdateColors( DC );
        SelectPalette( DC, palTemp, FALSE );
     end;
end;

procedure WMQueryNewPalette( var Msg: TWMQueryNewPalette; f: TForm; bm: TBitmap );
var
  DC: HDC;
  palTemp: HPalette;
  nRC: integer;
begin
  DC := f.Canvas.Handle;
  palTemp := SelectPalette( DC, bm.Palette, FALSE );
  nRC := RealizePalette( DC );
  SelectPalette( DC, palTemp, FALSE );
  if nRC > 0 then
     InvalidateRect( f.Handle, nil, FALSE );
  Msg.Result := 1;
end;

end.
